(** Main entry point for creating scoring matrix output for a sequence
    database *)

open Printf
open Arg

open Align
open Sequence
open Cmd_util
open Util
open Scoring
open Matrices
open Align_io
open Query_align

let query_keys = ref [||]

open Ckarg
(** Options for program execution *)
let prog_opts = []
let options = prog_opts @ Scoring.options @ Align.options @ Seqdb.options

let usg = 
  sprintf "usage: %s [options] -queries 'q1 [q2 ..]' -{filetype} seqfile -index indexfile [-{filetype} seqfile2 ...]"
    (Filename.basename Sys.executable_name)
let deffn s = failwith ("All arguments must have an option, "^s^" does not")


(* MAIN *)
let _ = begin
  let startuptime = Timing.interval () in
  parse options deffn usg;
  let seqDB = Seqdb.seqDB in
  let query_keys = !query_keys in
  let nq,nDB = Array.length query_keys, Hashtbl.length seqDB in
  ignore(Seqdb.verify_db seqDB);

  Scoring.default_score (seq_gsm_scores ~scale:(!Scoring.wgt) !Matrices.gmat);

  (* Allocate space for alignments/results *)
  let maxlen = Seqdb.maxlen_seqDB seqDB in
  let scores,_,_,_,_,_,_ = Align.allocate_max maxlen in

  let loadtime = Timing.interval () in
  printf "%d DB entries imported, %d queries selected\n" nDB nq;
  flush_all ();

  (* Loop over all queries *)
  for i=0 to nq-1 do
    let x = Hashtbl.find seqDB query_keys.(i) in
    for j=i+1 to nq-1 do
      let y = Hashtbl.find seqDB query_keys.(j) in 
      printf "%4d %4d: %s %s\n" i j query_keys.(i) query_keys.(j); flush stdout;
      Scoring.setup_scoring_mat scores x y;
      let fname = sprintf "%s-%s.smat" query_keys.(i) query_keys.(j) in
      let m, n = x.seqlen, y.seqlen in
      Util.write_matrix ~fname ~m ~n  "%8.3f " scores;
    done;
  done;

  let aligntime = Timing.interval () in
  printf "Timing\n%10s: %8.2f\n%10s: %8.2f\n%10s: %8.2f\n"
    "Startup" startuptime "DB Load" loadtime "Alignment" aligntime;
end
