open Printf
open Sequence
open Seqdb
open Util

let outfile = ref "seqdb.sdb"
let mode = ref ""
let printall = ref false
let extract_keys = ref [||]

open Ckarg
(** Options for program execution *)
let prog_opts = [
  "-mode","key","Mode of operation",Symbol(true,(fun s->mode:=s),[
      "create","Read sequence files, report stats, and output them to a file";
      "report","Only report statistics on the database";
    ]);
  "-report","","Change the mode to report",Unit(fun ()->mode:="report");
  "-o","file","Name of output file for storing a created database",Set_string(outfile);
  "-printall","","Print all sequence keys and their lenghths",Set(printall);
  "-extract","keys","Extract some of the database with given keys",
    String(fun s->extract_keys := Util.parse_strings s);
  "-dir","directory","Add the directory to search path for data files",String(Find.add_dir);
]

let options = prog_opts @ Seqdb.options

let usg = 
  sprintf "usage: %s [-mode m] [options] -{filetype} seqfile [-{filetype} seqfile2 ...]"
    (Filename.basename Sys.executable_name)
let deffn s = failwith ("All arguments must have an option, "^s^" does not")

let _ = begin
  parse options deffn usg;
  let seqDB, extract_keys = Seqdb.seqDB, !extract_keys in

  (* Perform extraction if necessary *)
  let seqDB = 
    let nextract = Array.length extract_keys in
    if nextract = 0 then seqDB
    else begin
      let newDB = Hashtbl.create nextract in
      for i=0 to nextract-1 do
	let key = extract_keys.(i) in
	try 
	  let seq = Hashtbl.find seqDB key in
	  Hashtbl.add newDB key seq
	with Not_found -> failwith (sprintf "Sequence %s not in database" key)
      done;
      newDB
    end
  in
	
  begin match !mode with
    "create" ->
      Seqdb.seqDB_report !printall seqDB;
      let f = open_out !outfile in
      Pervasives.output_value f seqDB;
  | "report" ->
      Seqdb.seqDB_report !printall seqDB;
  | s ->
      eprintf "Error: unknown mode %s\n" s;
  end;
end;
